@@ -1,7 +1,7 @@
 #!perl -w
 use strict;
 use Test;
-BEGIN { plan tests => 23 }
+BEGIN { plan tests => 24 }
 
 if (1) {
    print "\nRequired modules:\n";
@@ -26,7 +26,7 @@ if (1) {
 
 if (1 && $^O eq 'MSWin32') {
    print "\nWin32 optional modules, dependent on features used:\n";
-   foreach my $m ('Win32','Win32::TieRegistry', 'Win32API::Net', 'Win32::OLE') {
+   foreach my $m ('Win32','Win32::API','Win32::TieRegistry', 'Win32API::Net', 'Win32::OLE') {
      print "use $m\t";
      skip(!eval("use $m; 1"), 1);
    }
